# Generic Tcl Tips vis a vis Teaish


<a name="tclversions"></a>

# Hosting Multiple Tcl Versions

As testing teaish with multiple Tcl versions is important, my systems
use the following recipe for hosting multiple Tcl versions...

First, use [Fossil][] to clone [the Tcl source repository][tclsrc]:

>
```
$ mkdir -p ~/fossil/tcl
$ cd ~/fossil/tcl
$ fossil clone https://core.tcl-lang.org/tcl
```

Next, open a separate checkout for each interesting branch, e.g.:

>
```
$ mkdir 8.5
$ cd 8.5
$ fossil open ../tcl.fossil core-8-5-branch
```

For each of:

| **Version** | **Branch** |
|: 8.5       :|: core-8-5-branch :|
|: 8.6       :|: core-8-6-branch :|
|: 9.0       :|: core-9-0-branch :|
|: 9.1       :|: trunk (currently) :|

Build each one like so:

>
```
$ cd <VERSION>/unix
$ ./configure --prefix=$HOME/tcl/<VERSION> --disable-shared
              # Adjust the flags accordingly ^^^^^^^^
$ make all install
```

With those in place, we can use them with teaish:

>
```
$ ./configure --with-tcl=$HOME/tcl/9.0 ...
```

<a name="wrapper"></a>

# Teaish Wrapper Script

Invoking teaish with a [specific Tcl version](#tclversions) is a
common need, and has been simplified using a set of tiny wrapper
scripts:

>
```
$ cd ~/bin
$ cat teaish-with-tcl
#!/bin/sh
t="$1"
shift
set -x
exec $HOME/f/tcl/teaish/teaish/configure --with-tcl=$HOME/tcl/$t "$@"

$ cat teaish{85,86,90,91}
#!/bin/sh
exec `dirname $0`/teaish-with-tcl 8.5 "$@"
#!/bin/sh
exec `dirname $0`/teaish-with-tcl 8.6 "$@"
#!/bin/sh
exec `dirname $0`/teaish-with-tcl 9.0 "$@"
#!/bin/sh
exec `dirname $0`/teaish-with-tcl 9.1 "$@"
```

With those in place, we can configure any extension with:

>
```
$ cd /path/to/extension
$ teaish86 --flags...
```

Or, for an out-of-tree build:

>
```
$ cd /some/path
$ teaish86 --teaish-extension-dir=/path/to/extension --flags...
```

(Tip: `--teaish-extension-dir` can be abbreviated as `--t-e-d`.)

[tclsrc]: https://core.tcl-lang.org/tcl
[Fossil]: https://fossil-scm.org/home
